<?php
namespace Cylancer\Usertools\Controller;

use TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Domain\Model\FrontendUser;

/**
 * Class LoginController
 *
 * @package Cylancer\Usertools\Controller;
 */
class LoginController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    const TOKEN_KEY = 'token';

    const EMAIL_KEY = 'email';

    const USER_KEY = 'user';

    const ERRORS_KEY = 'errors';

    const MESSAGES_KEY = 'messages';

    const PASSWORD_KEY = 'password';

    /**
     * feUserUtility
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Utility\FeUserUtility
     */
    public $feUserUtility = null;

    /**
     * frontendUserRepository
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository
     */
    public $frontendUserRepository = null;

    /**
     * userRepository
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Domain\Repository\UserRepository
     */
    public $userRepository = null;

    /**
     * emailUtility
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Utility\EmailUtility
     */
    public $emailUtility = null;

    /**
     * messages container
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Domain\Model\Messages
     */
    public $messages = null;

    /**
     * frontendUserAuthentication
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \TYPO3\CMS\Frontend\Authentication\FrontendUserAuthentication
     */
    public $frontendUserAuthentication = null;

    /**
     * Show the login form
     */
    public function loginAction()
    {
        if (isset($_REQUEST['pass'])) {
            $uri = $this->uriBuilder->reset()->build();
            // debug ($uri);
           $this->redirectToURI($uri);
        }

        if ($this->feUserUtility->isLogged()) {
            $user = $this->feUserUtility->getCurrentUser();
            $this->view->assign(LoginController::USER_KEY, $user);
            $pageUid = $this->settings['redirectSuccess'];
            if ($pageUid != $this->getCurrentPageUid()) {
                $this->redirect('redirectToSuccessPage');
            }
        } else {
            $arguments = $this->request->getArguments();
            // debug ($arguments);
            $this->view->assign(ProfileController::MESSAGES_KEY, $arguments[LoginController::MESSAGES_KEY]);

            $this->view->assign('username', $arguments['username']);
            $this->view->assign('stayLoggedIn', $arguments['stayLoggedIn']);
        }
    }

    /**
     * Check the login data and logins the user with get parameter...
     */
    public function doLoginAction()
    {
        return;
        if ($this->feUserUtility->isLogged()) {
            $this->view->assign(LoginController::USER_KEY, $this->feUserUtility->getCurrentUser());
        } else {

            $username = $this->request->getArgument('username');
            $users = $this->frontendUserRepository->findByUsername($username);
            $password = $this->request->getArgument('password');
            $stayLoggedIn = $this->request->hasArgument('stayLoggedIn') && $this->request->getArgument('stayLoggedIn') === '1';

            $errors = array();
            if (count($users) == 1) {
                if ($this->checkPassword($password, $users[0]->getPassword())) {
                    // one account found :)
                    $this->loginUser($users[0], $password, $stayLoggedIn);
                    return;
                } else {
                    $errors[] = "passwordWrong";
                }
            } else {
                if ($this->settings['allowEmailLogin']) {
                    $users = $this->userRepository->findByEmail($username);
                    if (count($users) == 1) {
                        if ($this->checkPassword($this->request->getArgument('password'), $users[0]->getPassword())) {
                            // one account found :)
                            $this->loginUser($users[0], $password, $stayLoggedIn);
                            return;
                        } else {
                            $errors[] = "passwordWrong";
                        }
                    } else if (count($users) > 1) {
                        $errors[] = "tooManyAccountsfoundByEMail";
                    } else {
                        $errors[] = "accountNotfoundByUsernameAndEmail";
                    }
                } else if (count($users) > 1) {
                    $errors[] = "tooManyAccountsfoundByUsername";
                } else {
                    $errors[] = "accountNotfoundByUsername";
                }
            }
            $this->messages->addErrors($errors);
            $arguments = $this->request->getArguments();
            $arguments;
            // debug($arguments);

            $this->forward('login', null, null, $arguments);
        }
    }

    /**
     *
     * @return String Returns the storage pids...
     */
    private function getStoragePids()
    {
        return $this->configurationManager->getContentObject()->data['pages'];
    }

    /**
     *
     * @return String Retuns the pid of the current page
     */
    private function getCurrentPageUid()
    {
        return $this->configurationManager->getContentObject()->data['pid'];
    }

    /**
     * Logins the frontend user with get parameters...
     *
     * @param FrontendUser $user
     *            instance
     *            of the founded user
     * @param String $password
     *            contains
     *            the user password
     * @param boolean $stayLoggedIn
     *            the
     *            login countdown is longer if $stayLoggedIn is true
     */
    public function loginUser($user, $password, $stayLoggedIn = false)
    {
        $username = $user->getUsername();
        $u = $this->userRepository->findByUid($user->getUid());
        $u->setPasswordToken('');
        $this->userRepository->update($u);

//         $loginData = [
//             'uname' => $username,
//             'password' => $password
//         ];
        $_GET['user'] = $username;
        $_GET['pass'] = $password;
        $_GET['logintype'] = 'login';
        $_GET['permalogin'] = $stayLoggedIn ? 1 : 0;

        // protected function getAuthServices(string $subType, array $loginData, array $authInfo)
        /*
         * debug($this->frontendUserAuthentication->start());
         * debug($this->frontendUserAuthentication->checkAuthentication());
         * debug($this->frontendUserAuthentication->processLoginData($this->frontendUserAuthentication->getLoginFormData()));
         */

        $uri = $this->uriBuilder->reset()
            ->setArguments([
            'logintype' => 'login',
            'user' => $username,
            'pass' => $password,
            'pid' => $this->getStoragePids(),
            'permalogin' => $stayLoggedIn ? 1 : 0
        ])
            ->build();
        // debug ($uri);
        $this->redirectToURI($uri);
    }

    /**
     * Prepare and send a mail with a password change link
     */
    public function sendForgottenPasswordTokenAction()
    {
        // debug($this->request->getArgument('username'));
        $users = $this->userRepository->findByUsername($this->request->getArgument('username'));

        if (count($users) == 1) {
            $user = $users[0];
            // debug($user);
            if (! empty($user->getEmail())) {
                if ($this->sendForgottenPasswordTokenEMail($user)) {
                    $this->messages->addInfo('passwordChangeMailSuccessful');
                } else {
                    $this->messages->addError('passwordChangeMailFailed');
                }
            }
        }
        $arguments = $this->request->getArguments();
        $arguments[LoginController::MESSAGES_KEY] = $this->messages;
        $this->forward('login', null, null, $arguments);
    }

    /**
     * Displays a from to send a forgotten password token email.
     */
    public function forgottenPasswordAction()
    {}

    /**
     * Send a mail with a password change link
     *
     * @param FrontendUser $user
     *            is
     *            an instance of an frondend user
     */
    public function sendForgottenPasswordTokenEMail(FrontendUser $user)
    {
        // sendTemplateEmail(array $recipient, array $sender, $subject, $templateName, $extensionName, array $variables = array()) {
        $token = sha1(uniqid($user->getEmail()));
        $user->setPasswordToken($token);
        $this->userRepository->update($user);

        return $this->emailUtility->sendTemplateEmail([
            $user->getEmail()
        ], [
            $this->settings['sender'] => $this->settings['senderName']
        ], \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('forgottenPassword.mail.subject', 'Usertools'), 'ForgottenPasswordEmail', $this->request->getControllerExtensionName(), [
            ProfileController::TOKEN_KEY => $token,
            ProfileController::USER_KEY => $user,
            ProfileController::TARGET_PAGE => $this->getCurrentPageUid()
        ]);
    }

    /**
     * Resets the forgotten password
     *
     * @param \Cylancer\Usertools\Domain\Model\Messages $messages
     *            contains error und info messages
     * @param String $password
     *            is a container with the password and a repeat password
     */
    public function changeForgottenPasswordAction(\Cylancer\Usertools\Domain\Model\Messages $messages = null, \Cylancer\Usertools\Domain\Model\Password $password = null)
    {
        $token = $this->request->hasArgument(LoginController::TOKEN_KEY) ? $this->request->getArgument(LoginController::TOKEN_KEY) : null;
        $tokenWrong = false;
        if ($password == null) {
            $password = $this->objectManager->get(\Cylancer\Usertools\Domain\Model\Password::class);
        } else {
            $user = null;
            if (! $this->request->hasArgument(LoginController::TOKEN_KEY)) {
                $tokenWrong = true;
                $this->messages->addError('tokenWrong');
            } else {
                $users = $this->userRepository->findByPasswordToken($this->request->getArgument(LoginController::TOKEN_KEY));
                if (count($users) != 1) {
                    $tokenWrong = true;
                    $this->messages->addError('tokenWrong');
                } else {
                    $user = $users[0];
                    if ($password != null) {
                        $this->messages->addErrors($this->validatePassword($password, $user));
                    }
                }
            }

            if (! $this->messages->hasErrors()) {
                $token = '';
                $tokenWrong = true;
                $user->setPassword($this->getHashedPassword($password->getPassword()));
                $user->setPasswordToken($token);
                $this->userRepository->update($user);
                $this->messages->addInfo('passwordChangeSuccessful');
                $password = null;
            } else {
                $password->setConfirmPassword("");
            }
        }
        $this->view->assign(ProfileController::PASSWORD_KEY, $password);
        $this->view->assign(ProfileController::MESSAGES_KEY, $this->messages);
        $this->view->assign(ProfileController::TOKEN_KEY, $token);
        $this->view->assign(ProfileController::TOKEN_KEY . 'Wrong', $tokenWrong);
    }

    /**
     * Checks the new password
     *
     * @param String $password
     *            is a container with the password and a repeat password
     *            param $user is an instance of the frondend user
     */
    private function validatePassword(\Cylancer\Usertools\Domain\Model\Password $password, \Cylancer\Usertools\Domain\Model\User $user)
    {
        $errors = array();

        if ($this->checkPassword($password->getPassword(), $user->getPassword())) {
            $errors[] = 'newPasswordIsOldPassword';
        }

        if ($password->getPassword() !== $password->getConfirmPassword()) {
            $errors[] = 'confirmPasswordIsNotEquals';
        }

        if (! preg_match('/' . ProfileController::PASSWORD_RULES . '/', $password->getPassword())) {
            $errors[] = 'passwordInvalid';
        }
        return $errors;
    }

    /**
     * Logout and redirect to the new page...
     */
    public function logoutAction()
    {
        $pageUid = $this->settings['redirectSuccess'];
        $uriBuilder = $this->uriBuilder;
        $uri = $uriBuilder->reset()
            ->setTargetPageUid($pageUid)
            ->setArguments([
            'logintype' => 'logout'
        ])
            ->build();
        $this->redirectToURI($uri, 0, 404);
    }

    /**
     * Redirects to the new page...
     */
    public function redirectToSuccessPageAction()
    {
        $pageUid = $this->settings['redirectSuccess'];
        $uriBuilder = $this->uriBuilder;
        $uri = $uriBuilder->reset()
            ->setTargetPageUid($pageUid)
            ->build();
        $this->redirectToURI($uri);
    }

    /**
     * Check password
     *
     * @return boolean
     */
    public function checkPassword($password, $passwordHash)
    {
        return GeneralUtility::makeInstance(PasswordHashFactory::class)->get($passwordHash, 'FE')->checkPassword($password, $passwordHash);
    }

    /**
     * Get the hashed password
     *
     * @return null|string
     */
    private function getHashedPassword($password)
    {
        return GeneralUtility::makeInstance(PasswordHashFactory::class)->getDefaultHashInstance('FE')->getHashedPassword($password);
    }
}
