<?php
namespace Cylancer\Usertools\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use Cylancer\Usertools\Property\TypeConverter\UploadedFileReferenceConverter;
use TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory;

/**
 * Class LoginController
 *
 * @package Cylancer\Usertools\Controller;
 */
class ProfileController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    const TOKEN_KEY = 'token';

    const EMAIL_KEY = 'email';

    const USER_KEY = 'user';

    const ERRORS_KEY = 'errors';

    const MESSAGES_KEY = 'messages';

    const PASSWORD_KEY = 'password';

    const TARGET_PAGE = 'targetPage';

    const PASSWORD_RULES = '^(?=(?:[^A-Z]*[A-Z]){1})(?=(?:[^0-9]*[0-9]){1})(?=(?:[^!#+-_"\/§$%&\()\[\]\{\}]*[!#+-_"\/§$%&\()\[\]\{\}]){1}).{8,}$';

    /**
     * messages container
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Domain\Model\Messages
     */
    public $messages = null;

    /**
     * frontendUserRepository
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository
     */
    public $frontendUserRepository = null;

    /**
     * userRepository
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Domain\Repository\UserRepository
     */
    public $userRepository = null;

    /**
     * feUserUtility
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Utility\FeUserUtility
     */
    public $feUserUtility = null;

    /**
     * emailUtility
     *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Cylancer\Usertools\Utility\EmailUtility
     */
    public $emailUtility = null;

    private $previousPicture = null;

    /**
     * Set TypeConverter option for image upload
     */
    public function initializeDoEditProfileAction()
    {
        $this->setTypeConverterConfigurationForImageUpload(ProfileController::USER_KEY);
        if ($this->feUserUtility->isLogged()) {
            $u = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
            if ($u->getPicture() != null) {
                $fileReference = \TYPO3\CMS\Core\Resource\ResourceFactory::getInstance()->getFileReferenceObject($u->getPicture()
                    ->getUid());
                $this->previousPicture = $fileReference;
            }
        }
    }

    public function redirectToSuccessPageAction()
    {
        $pageUid = $this->settings['redirectSuccess'];
        $uriBuilder = $this->uriBuilder;
        $uri = $uriBuilder->reset()
            ->setTargetPageUid($pageUid)
            ->build();
        $this->redirectToURI($uri, 0, 404);
    }

    /**
     *
     * @param \Cylancer\Usertools\Domain\Model\User $user
     *
     */
    public function doEditProfileAction(\Cylancer\Usertools\Domain\Model\User $user = null)
    {
        if ($user == null) {
            $this->forward('editProfile');
        } else if ($this->feUserUtility->isLogged()) {
            if ($this->feUserUtility->getCurrentUserUid() != $user->getUid()) {
                $this->messages->addError('onlyCurrentUserIsEditable');
            }
        } else {
            $this->messages->addError('notLogged');
        }
        if (! $this->messages->hasErrors()) {
            if ($this->previousPicture != null && $this->previousPicture->getOriginalFile()->getIdentifier() != $user->getPicture()
                ->getOriginalResource()
                ->getOriginalFile()
                ->getIdentifier()) {
                $this->previousPicture->getOriginalFile()->delete();
            }
            $this->userRepository->update($user);
        }
        $this->forward('editProfile', null, null, [
            ProfileController::MESSAGES_KEY => $this->messages
        ]);
    }

    /**
     *
     * @param \Cylancer\Usertools\Domain\Model\User $user
     */
    public function deleteUserPictureAction(\Cylancer\Usertools\Domain\Model\User $user = null)
    {
        if ($this->feUserUtility->isLogged()) {
            if ($this->feUserUtility->getCurrentUserUid() != $user->getUid()) {
                $this->messages->addError('onlyCurrentUserIsEditable');
            }
        } else {
            $this->messages->addError('notLogged');
        }

        if (! $this->messages->hasErrors()) {
            if ($user->getPicture() != null) {
                $fuid = $user->getPicture()->getUid();
                $fileReference = \TYPO3\CMS\Core\Resource\ResourceFactory::getInstance()->getFileReferenceObject($fuid);
                $user->setPicture(null);
                $this->userRepository->update($user);
                $fileReference->getOriginalFile()->delete();
            }
        }
        $this->forward('editProfile', null, null, [
            ProfileController::MESSAGES_KEY => $this->messages
        ]);
    }

    /**
     * Show the user settings
     */
    public function editProfileAction()
    {
        if ($this->request->hasArgument(ProfileController::MESSAGES_KEY)) {
            $this->messages = $this->request->getArgument(ProfileController::MESSAGES_KEY);
        }
        // debug($this->messages);

        if ($this->feUserUtility->isLogged()) {
            $user = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
            $this->view->assign(ProfileController::USER_KEY, $user);
        } else {
            $this->messages->addError('notLogged');
        }
        $this->view->assign(ProfileController::MESSAGES_KEY, $this->messages);
    }

    /**
     * Sets the type converter for the image upload...
     */
    private function setTypeConverterConfigurationForImageUpload($argumentName)
    {
        $uploadConfiguration = [
            UploadedFileReferenceConverter::CONFIGURATION_ALLOWED_FILE_EXTENSIONS => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'],
            UploadedFileReferenceConverter::CONFIGURATION_UPLOAD_FOLDER => '/fileadmin/userPics'
        ];
        $newExampleConfiguration = $this->arguments[$argumentName]->getPropertyMappingConfiguration();
        $newExampleConfiguration->forProperty('picture')->setTypeConverterOptions('Cylancer\\Usertools\\Property\\TypeConverter\\UploadedFileReferenceConverter', $uploadConfiguration);
    }

    /**
     *
     * @param \Cylancer\Usertools\Domain\Model\Messages $messages
     * @param \Cylancer\Usertools\Domain\Model\Password $password
     */
    public function changePasswordFormAction(\Cylancer\Usertools\Domain\Model\Messages $messages = null, \Cylancer\Usertools\Domain\Model\Password $password = null)
    {
        if ($this->request->hasArgument('messages') != null) {
            $this->messages = $this->request->getArgument('messages');
        }
        if (! $this->feUserUtility->isLogged()) {
            $this->messages->addError('notLogged');
        }

        if ($password == null) {
            $password = $this->objectManager->get(\Cylancer\Usertools\Domain\Model\Password::class);
            $this->view->assign(ProfileController::PASSWORD_KEY, $password);
        }

        $this->view->assign(ProfileController::MESSAGES_KEY, $this->messages);
    }

    /**
     *
     * @param \Cylancer\Usertools\Domain\Model\Password $password
     */
    public function changePasswordAction(\Cylancer\Usertools\Domain\Model\Password $password)
    {
        if ($this->feUserUtility->isLogged()) {
            $this->messages->addErrors($this->validatePassword($password));
        } else {
            $this->messages->addError('notLogged');
        }

        if (! $this->messages->hasErrors()) {
            $user = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
            $user->setPassword($this->getHashedPassword($password->getPassword()));
            $this->userRepository->update($user);
            $this->messages->addInfo('passwordChangeSuccessful');
            $password = null;
        } else {
            $password->setConfirmPassword("");
        }
        $this->forward('changePasswordForm', null, null, [
            ProfileController::PASSWORD_KEY => $password,
            ProfileController::MESSAGES_KEY => $this->messages
        ]);
    }

    /**
     * Checks the password of the current front end user
     */
    private function validatePassword(\Cylancer\Usertools\Domain\Model\Password $password)
    {
        $errors = array();
        $user = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
        if ($this->checkPassword($password->getPassword(), $user->getPassword())) {
            $errors[] = 'newPasswordIsOldPassword';
        }

        if ($password->getPassword() !== $password->getConfirmPassword()) {
            $errors[] = 'confirmPasswordIsNotEquals';
        }

        if (! preg_match('/' . ProfileController::PASSWORD_RULES . '/', $password->getPassword())) {
            $errors[] = 'passwordInvalid';
        }
        return $errors;
    }

    /**
     * Displays the change email form
     */
    public function changeEmailFormAction()
    {
        if ($this->request->hasArgument(ProfileController::MESSAGES_KEY)) {
            $this->messages = $this->request->getArgument(ProfileController::MESSAGES_KEY);
        }

        if ($this->request->hasArgument(ProfileController::EMAIL_KEY)) {
            $email = $this->request->getArgument(ProfileController::EMAIL_KEY);
        } else {
            $email = $this->objectManager->get(\Cylancer\Usertools\Domain\Model\Email::class);
        }

        if ($this->feUserUtility->isLogged()) {
            $user = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
        } else {
            $this->messages->addError('notLogged');
        }
        $this->view->assign(ProfileController::EMAIL_KEY, $email);
        $this->view->assign(ProfileController::USER_KEY, $user);
        $this->view->assign(ProfileController::MESSAGES_KEY, $this->messages);
    }

    public function prepareEmailChangeAction(\Cylancer\Usertools\Domain\Model\Email $email)
    {
        $errors = array();
        if ($this->feUserUtility->isLogged()) {
            $user = $this->userRepository->findByUid($this->feUserUtility->getCurrentUserUid());
            // No email
            if (empty($email->getEmail())) {
                $errors[] = 'newEmailEmpty';
            } // E-Mail Format
            elseif (! filter_var($email->getEmail(), FILTER_VALIDATE_EMAIL)) {
                $errors[] = 'newEmailInvalid';
            } elseif ($email->getEmail() === $user->getEmail()) {
                $errors[] = 'newEmailIsOldEmail';
            } elseif (count($this->userRepository->findByEmail($email->getEmail())) > 0) {
                $errors[] = 'newEmailExists';
            }
        } else {
            $errors[] = 'notLogged';
        }
        $this->messages->addErrors($errors);

        if (! $this->messages->hasErrors()) {
            $user->setNewEmail($email->getEmail());
            $emailHash = sha1(uniqid($email->getEmail()));
            $user->setNewEmailToken($emailHash);
            $this->userRepository->update($user);

            // EmailUtility#sendTemplateEmail(array $recipient, array $sender, $subject, $templateName, $extensionName, array $variables = array()) {
            if ($this->emailUtility->sendTemplateEmail([
                $email->getEmail() => $user->getName()
            ], [
                $this->settings['sender'] => $this->settings['senderName']
            ], \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('changeEmail.mail.confirm.subject', 'Usertools'), ProfileController::EMAIL_KEY, $this->request->getControllerExtensionName(), [
                ProfileController::TOKEN_KEY => $emailHash,
                ProfileController::USER_KEY => $user,
                ProfileController::TARGET_PAGE => $this->settings['redirectSuccess']
            ])) {
                $this->messages->addInfo('confirmEMailMailSuccessful');
            } else {
                $this->messages - addError('confirmEMailMailFailed');
            }

            $email = '';
        }
        $this->forward('changeEmailForm', null, null, [
            ProfileController::EMAIL_KEY => $email,
            ProfileController::MESSAGES_KEY => $this->messages
        ]);
    }

    /**
     * Confirms the new email address
     */
    public function confirmNewEmailAction()
    {
        if ($this->request->hasArgument(ProfileController::TOKEN_KEY) && $this->request->hasArgument(ProfileController::EMAIL_KEY)) {

            $token = $this->request->getArgument(ProfileController::TOKEN_KEY);
            $email = $this->request->getArgument(ProfileController::EMAIL_KEY);
            $user = null;

            $users = $this->userRepository->findByNewEmailToken($token);

            if (count($users) == 1) {
                foreach ($this->userRepository->findByNewEmailToken($token) as $u) {
                    if ($u->getNewEmail() === $email) {
                        $user = $u;
                        break;
                    }
                }
                ;
            }

            if ($user != null) {
                $user->setNewEmail('');
                $user->setNewEmailToken('');
                $user->setEmail($email);
                $this->userRepository->update($user);
                $this->view->assign('user', $user);
            } else {
                $this->messages->addError('tokenEmailCombinationNotFound');
            }
        } else {
            $this->messages->addError('invalidLink');
        }

        $this->view->assign(ProfileController::MESSAGES_KEY, $this->messages);
    }

    /**
     * Get the hashed password
     *
     * @return null|string
     */
    private function getHashedPassword($password)
    {
        return GeneralUtility::makeInstance(PasswordHashFactory::class)->getDefaultHashInstance('FE')->getHashedPassword($password);
    }

    /**
     * Check password
     *
     * @return null|string
     */
    public function checkPassword($password, $passwordHash)
    {
        return GeneralUtility::makeInstance(PasswordHashFactory::class)->get($passwordHash, 'FE')->checkPassword($password, $passwordHash);
    }
}
